# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'easy_type'
require 'puppet/provider/exec/windows'

Puppet::Type.type(:validation).provide :auditpol, :parent => Puppet::Type::Exec::ProviderWindows do
  confine 'os.name' => :windows
  include EasyType::Provider

  @output = ''

  desc 'Execute a validation on the Windows Audit Policies'

  mk_resource_methods

  def validate_run?
    subcat = []
    `auditpol /list /subcategory:*`&.split("\n")&.drop(1)&.each do |line|
      next if line[0] =~ /\S/

      subcat << line.strip
    end
    subcat.include?(resource[:check])
  end

  def execute_check
    @output = run(resource.check)
  end

  def report
    if resource.fail_message != :absent
      message = resource[:fail_message] % {output: @output}
    else
      message = "Validation #{name}  failed; expected: '#{resource.expected_output}' got : '\n#{@output}'"
    end
    message = "#{resource.path}: #{message}"
    case resource.report_as
    when /warning/i
      Puppet.warning message
    when /error/i
      raise Puppet::Error, message
    when /command/i
      # Issue a waning and execute the fail_command
      Puppet.warning message
      final_command = resource.fail_command % {output: @output}
      run(final_command, true, resource.parameters[:fail_command].sensitive).first.to_s
    when /none/i
      Puppet.debug message
    else
      raise "#{resource.report_as} is not a valid value"
    end
  end

  def run(subcategory)
    begin
      Puppet::Util::Execution.execute("auditpol /get /subcategory:\"#{subcategory}\"", :failonfail => true)&.split("\n")&.last.gsub(subcategory,'').strip
    rescue
      Puppet.debug "Failed to find Audit Policy for subcategory #{subcategory}"
      raise Puppet::Error.new "Failed to find Audit Policy for subcategory #{subcategory}"
    end
  end
end
