# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'easy_type'

Puppet::Type.type(:validation).provide :facter do
  include EasyType::Provider

  @output = ''

  desc 'Execute a validation on a facter value'

  mk_resource_methods

  def validate_run?
    [String, Hash, Array].include?(factval(resource[:check]).class)
  end

  def factval(check)
    Facter.value(check.split('.').first)
  end

  def execute_check
    @output = facter_value(resource.check)
  end

  def report
    message = if resource.fail_message != :absent
                format(resource[:fail_message], :output => @output)
              else
                "Validation #{name}  failed; expected: '#{resource.expected_output}' got : '\n#{@output}'"
              end
    message = "#{resource.path}: #{message}"
    case resource.report_as
    when /warning/i
      Puppet.warning message
    when /error/i
      raise Puppet::Error, message
    when /command/i
      # Issue a waning and execute the fail_command
      Puppet.warning message
      final_command = format(resource.fail_command, :output => @output)
      run(final_command, true, resource.parameters[:fail_command].sensitive).first.to_s
    when /none/i
      Puppet.debug message
    else
      raise "#{resource.report_as} is not a valid value"
    end
  end

  def facter_value(check)
    fail 'Use dot notation as argument to facter provider' if check.include?('[') || check.include?(']')

    fact = check.split('.').first
    fact_keys = check.split('.').drop(1)
    full_fact = Facter.value(fact)
    output = full_fact.dig(*fact_keys)
    output = output.nil? ? '' : output
    Puppet.debug "Fact \"#{check}\" value: #{output}"
    output
  end
end
